/*global JQueryEventObject*/
/* Rangeslider.ts

	Purpose:

	Description:

	History:
		Thu Sep 12 11:19:05 CST 2019, Created by jameschu

Copyright (C) 2019 Potix Corporation. All Rights Reserved.

This program is distributed under LGPL Version 2.1 in the hope that
it will be useful, but WITHOUT ANY WARRANTY.
*/
(() => {
let _zkf: () => void;
/**
 * A Rangeslider.
 *
 * @author jameschu
 * @since 9.0.0
 */
zkex.slider.Rangeslider = zk.$extends(zul.Widget, {
    _min: 0,
    _max: 100,
    _step: 1,
    _orient: 'horizontal',
    _marks: '20',
    _tooltipVisible: false,
    buttonsFireChangeBlocked_: true,

    $define: {
        /**
         * Returns the minimum value of the rangeslider.
         * <p>Default: 0.
         * @return int
         */
        /**
         * Sets the minimum value of the rangeslider.
         * @param int minimum value
         */
        min: _zkf = function () {
            if (this.desktop) {
                this.redrawMarks_();
                this.updatePosByValues_();
            }
        },
        /**
         * Returns the maximum value of the rangeslider.
         * <p>Default: 100.
         * @return int
         */
        /**
         * Sets the maximum value of the rangeslider.
         * @param int maximum value
         */
        max: _zkf,
        /**
         * Returns marks of the rangeslider.
         * @return String
         */
        /**
         * Sets the marks information for displaying value marks.
         * <p>Default: '20'.
         *
         * <p>If there is only a number value, for example: 20.
         * The value marks would be displayed every 20 start from the minimum value. (if min is 0, then display '0 20 40 ...')
         *
         * <p> Another options for the marks can be specified as follows:
         *  <pre><code>&lt;rangeslider marks='10:10%, 20:20%, 50:50%, 80:80%'</code></pre>
         * <p> It means that each value mark could be displayed in different text.
         *
         * @param String marks the value marks displaying information
         */
        marks() {
            if (this.desktop)
                this.redrawMarks_();
        },
        /**
         * Returns the step of rangesliderslider.
         * When the user drags the slider buttons, the buttons would increase/decrease value by the step count.
         * @return int
         */
        /**
         * Sets the step of rangeslider
         * <p>Default: 1
         * @param int step
         */
        step: null,
        /** Returns the orient.
         * <p>Default: 'horizontal'.
         * @return String
         */
        /** Sets the orient.
         * <p>Default : 'horizontal'
         * @param String orient either 'horizontal' or 'vertical'.
         */
        orient() {
            if (this.desktop)
                this.rerender();
        },
        /**
         * Returns whether it is disabled.
         * <p>
         * Default: false.
         * @return boolean
         */
        /**
         * Sets whether it is disabled.
         * @param boolean disabled
         */
        disabled(disabled: boolean) {
            if (this.desktop) {
                jq(this).toggleClass(this.$s('disabled'), disabled);
                if (disabled) {
                    if (!this._tooltipVisible)
                        this.setTooltipVisible(false);
                }
            }
        },
        /**
         * Returns whether it is tooltipVisible.
         * If true, the tooltips of the slider buttons would be always visible.
         * Default: false.
         * @return boolean
         */
        /**
         * Sets whether it is tooltipVisible.
         * If true, the tooltips of the slider buttons would be always visible.
         * @param boolean tooltipVisible
         */
        tooltipVisible(visible: boolean) {
            if (this.desktop) {
                for (let w = this.firstChild; w; w = w.nextSibling)
                    w.setTooltipVisible(visible);
            }
        }
    },
    bind_() {
        this.$supers(zkex.slider.Rangeslider, 'bind_', arguments);
        //prepare marks
        this.redrawMarks_();

        //update pos
        this.updatePosByValues_();
    },

    unbind_() {
        this.$supers(zkex.slider.Rangeslider, 'unbind_', arguments);
    },

    domClass_() {
        let scls = this.$supers('domClass_', arguments);
        if (this.isVertical())
            scls += ' ' + this.$s('vertical');
        else
            scls += ' ' + this.$s('horizontal');
        if (this._disabled)
            scls += ' ' + this.$s('disabled');
        return scls;
    },

    doMouseDown_(evt: zk.Event) {
        if (this._disabled) return;
        const btns = this.firstChild;
        if (!btns) return;
        let jqTargetDom = jq(evt.domTarget),
            isMarkReached = jqTargetDom.hasClass(this.$s('mark-dot')) || jqTargetDom.hasClass(this.$s('mark-label'));
        if (isMarkReached || jqTargetDom.hasClass(this.$s('track')) || jqTargetDom.hasClass(btns.$s('area'))) {
            let startBtn = btns.$n('start-btn'),
                endBtn = btns.$n('end-btn'),
                isVert = this.isVertical(),
                ofDim = isVert ? 'top' : 'left',
                btn1Offset = parseInt(startBtn.style[ofDim]),
                btn2Offset = parseInt(endBtn.style[ofDim]),
                targetBtn = startBtn,
                oldValues = btns._values.slice(),
                realOffset;
            if (isMarkReached) {
                let markVal = jqTargetDom.parent().data('value'),
                    step = this._step;
                if ((markVal - this._min) % step == 0) {
                    realOffset = parseInt(jqTargetDom[0].parentNode.style[ofDim]);
                    if (realOffset > (btn1Offset + (btn2Offset - btn1Offset) / 2))
                        targetBtn = endBtn;
                    btns._values[targetBtn == endBtn ? 1 : 0] = markVal;
                } else
                    isMarkReached = false; // due to step
            }

            if (!isMarkReached) {
                const ofIndex = isVert ? 1 : 0,
                    mouseTrackOffset = (isVert ? evt.pageY : evt.pageX) - zk(this.$n('track')).revisedOffset()[ofIndex],
                    totalDim = this.getSliderSize_();
                if (mouseTrackOffset > ((btn1Offset + (btn2Offset - btn1Offset) / 2) / 100 * totalDim))
                    targetBtn = endBtn;
                realOffset = 100 * btns._realSlidePos(targetBtn, [mouseTrackOffset, mouseTrackOffset], false)[ofIndex] / totalDim;
            }

            targetBtn.style[ofDim] = realOffset + '%';
            btns.syncArea();
            //fire onChange
            this.fireValuesChange_(btns, oldValues, btns._values);

            // change active target to button
            this.triggerBtnMouseDown_(evt.domEvent, btns, targetBtn, targetBtn == startBtn);
        }
        this.$supers('doMouseDown_', arguments);
    },

    triggerBtnMouseDown_(evt: JQueryEventObject, sliderBtns: zk.Widget, targetBtn: HTMLElement, isStartButton: boolean) {
        evt.delegateTarget = evt.target = evt.currentTarget = targetBtn;
        if (isStartButton)
            sliderBtns._dragStartBtn._mousedown(evt);
        else
            sliderBtns._dragEndBtn._mousedown(evt);
    },

    /**
     * Returns whether it is a vertical slider.
     * @return boolean
     */
    isVertical() {
        return 'vertical' == this._orient;
    },

    getSliderSize_() {
        return jq(this.$n('track'))[this.isVertical() ? 'height' : 'width']();
    },

    redrawMarks_() {
        //clear
        const jqMarks = jq(this.$n('marks'));
        jqMarks.html('');
        const marks = this._marks;
        if (!marks) return;
        const marksInfo = [],
            min = this._min,
            max = this._max;
        if (!isNaN(parseInt(marks))) { // pure number string
            let markStep = parseInt(marks),
                markVal = min;
            while (markVal <= max) {
                marksInfo.push([markVal, markVal]);
                markVal += markStep;
            }
            if (markVal < (max + markStep))
                marksInfo.push([max, max]);
        } else {
            for (var markVal in marks)
                marksInfo.push([markVal, marks[markVal]]);
        }
        const isVert = this.isVertical(),
            offStr = isVert ? 'top' : 'left',
            dotCls = this.$s('mark-dot'),
            labelCls = this.$s('mark-label'),
            markCls = this.$s('mark'),
            sliderValue = max - min,
            l = marksInfo.length;

        for (let i = 0; i < l; i++) {
            if (marksInfo[i][1] - min < 0) continue; //skip invalid
            const jqMark = jq('<div></div>'),
                jqDot = jq('<div></div>'),
                jqLabel = jq('<div></div>'),
                markValue = parseInt(marksInfo[i][0]);
            jqDot.addClass(dotCls);
            jqLabel.addClass(labelCls);
            jqLabel.html(marksInfo[i][1]);
            jqMark.addClass(markCls);
            jqMark.css(offStr, ((markValue - min) / sliderValue * 100) + '%');
            jqMark.data('value', markValue);
            jqMark.append(jqDot).append(jqLabel);
            jqMarks.append(jqMark);
        }
        this.updateActiveMarks_();
    },

    updateActiveMarks_() {
        const btns = this.firstChild;
        if (!btns) return;
        //update marks in area
        const jqMarks = jq(this).find('.' + this.$s('mark')),
            activeCls = this.$s('mark-active'),
            values = btns.getValues();
        jqMarks.removeClass(activeCls);
        let i = 0, l = jqMarks.length;
        for (; i < l; i++) {
            const markOffset = jqMarks.eq(i).data('value');
            if (markOffset >= values[0] && markOffset <= values[1])
                jqMarks.eq(i).addClass(activeCls);
        }
    },

    fireValuesChange_(wgt: zk.Widget, oldVals: number[], newVals: number[]) {
        if (oldVals[0] != newVals[0] || oldVals[1] != newVals[1]) {
            if (!this.buttonsFireChangeBlocked_)
                wgt.fire('onRangeValueChange', {values: newVals});
            this.afterBtnsValuesChange_(wgt);
        }
    },

    updatePosByValues_(btns?) {
        // size change
        const min = this._min,
            isVert = this.isVertical(),
            totalValue = this._max - min,
            btnDimStr = isVert ? 'top' : 'left';

        for (let w = this.firstChild; w; w = w.nextSibling) {
            if (btns && btns !== w) continue;
            const vals = w.getValues(),
                startBtn = w.$n('start-btn'),
                endBtn = w.$n('end-btn');

            startBtn.style[btnDimStr] = (100 * (vals[0] - min) / totalValue) + '%';
            endBtn.style[btnDimStr] = (100 * (vals[1] - min) / totalValue) + '%';
            w.syncArea();
        }
    },

    fixOtherChildrenValues_(wgt: zk.Widget) {
        const btns = this.firstChild;
        if (!btns) return;
        this.updatePosByValues_();
    },

    afterBtnsValuesChange_(btns: zk.Widget) {
        this.fire('onRangeValueChange', {values: btns.getValues()});
    },
    updateOtherChildrenPos_() {
        //do nothing when only one sliderbuttons
    }
});
})();